@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: connect.c,v 1.65 2003/10/18 20:34:33 bagder Exp $
+ * $Id: connect.c,v 1.82 2004/03/17 12:46:45 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -26,8 +26,12 @@
 #ifndef WIN32
 /* headers for non-win32 */
 #include <sys/time.h>
-#include <sys/socket.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
 #include <sys/ioctl.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -48,6 +52,13 @@
 #include <stdlib.h> /* required for free() prototype, without it, this crashes
                        on macos 68K */
 #endif
+#if (defined(HAVE_FIONBIO) && defined(__NOVELL_LIBC__))
+#include <sys/filio.h>
+#endif
+#if (defined(NETWARE) && defined(__NOVELL_LIBC__))
+#undef in_addr_t
+#define in_addr_t unsigned long
+#endif
 #ifdef	VMS
 #include <in.h>
 #include <inet.h>
@@ -64,7 +75,6 @@
 #endif
 
 #ifdef WIN32
-#define HAVE_IOCTLSOCKET
 #include <windows.h>
 #define EINPROGRESS WSAEINPROGRESS
 #define EWOULDBLOCK WSAEWOULDBLOCK
@@ -76,12 +86,15 @@
 #include "urldata.h"
 #include "sendf.h"
 #include "if2ip.h"
+#include "connect.h"
 
 /* The last #include file should be: */
 #ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
+static bool verifyconnect(curl_socket_t sockfd);
+
 int Curl_ourerrno(void)
 {
 #ifdef WIN32
@@ -98,7 +111,7 @@ int Curl_ourerrno(void)
  *  Set the socket to either blocking or non-blocking mode.
  */
 
-int Curl_nonblock(int sockfd,    /* operate on this */
+int Curl_nonblock(curl_socket_t sockfd,    /* operate on this */
                   int nonblock   /* TRUE or FALSE */)
 {
 #undef SETBLOCK
@@ -162,13 +175,19 @@ int Curl_nonblock(int sockfd,    /* operate on this */
  * 2    select() returned with an error condition
  */
 static
-int waitconnect(int sockfd, /* socket */
-                int timeout_msec)
+int waitconnect(curl_socket_t sockfd, /* socket */
+                long timeout_msec)
 {
   fd_set fd;
   fd_set errfd;
   struct timeval interval;
   int rc;
+#ifdef mpeix
+  /* Call this function once now, and ignore the results. We do this to
+     "clear" the error state on the socket so that we can later read it
+     reliably. This is reported necessary on the MPE/iX operating system. */
+  verifyconnect(sockfd);
+#endif
 
   /* now select() until we get connect or timeout */
   FD_ZERO(&fd);
@@ -200,22 +219,22 @@ int waitconnect(int sockfd, /* socket */
 }
 
 static CURLcode bindlocal(struct connectdata *conn,
-                          int sockfd)
+                          curl_socket_t sockfd)
 {
 #ifdef HAVE_INET_NTOA
-
+  bool bindworked = FALSE;
   struct SessionHandle *data = conn->data;
 
   /*************************************************************
    * Select device to bind socket to
    *************************************************************/
   if (strlen(data->set.device)<255) {
-    struct sockaddr_in sa;
     struct Curl_dns_entry *h=NULL;
     size_t size;
     char myhost[256] = "";
     in_addr_t in;
     int rc;
+    bool was_iface = FALSE;
 
     /* First check if the given name is an IP address */
     in=inet_addr(data->set.device);
@@ -227,24 +246,24 @@ static CURLcode bindlocal(struct connectdata *conn,
        */
       rc = Curl_resolv(conn, myhost, 0, &h);
       if(rc == 1)
-        rc = Curl_wait_for_resolv(conn, &h);
+        (void)Curl_wait_for_resolv(conn, &h);
 
+      if(h)
+        was_iface = TRUE;
     }
-    else {
-      if(strlen(data->set.device)>1) {
-        /*
-         * This was not an interface, resolve the name as a host name
-         * or IP number
-         */
-        rc = Curl_resolv(conn, data->set.device, 0, &h);
-        if(rc == 1)
-          rc = Curl_wait_for_resolv(conn, &h);
 
-        if(h)
-          /* we know data->set.device is shorter than the myhost array */
-          strcpy(myhost, data->set.device);
+    if(!was_iface) {
+      /*
+       * This was not an interface, resolve the name as a host name
+       * or IP number
+       */
+      rc = Curl_resolv(conn, data->set.device, 0, &h);
+      if(rc == 1)
+        (void)Curl_wait_for_resolv(conn, &h);
 
-      }
+      if(h)
+        /* we know data->set.device is shorter than the myhost array */
+        strcpy(myhost, data->set.device);
     }
 
     if(! *myhost) {
@@ -254,11 +273,37 @@ static CURLcode bindlocal(struct connectdata *conn,
          hostent_buf,
          sizeof(hostent_buf));
       */
+      failf(data, "Couldn't bind to '%s'", data->set.device);
       return CURLE_HTTP_PORT_FAILED;
     }
 
     infof(data, "We bind local end to %s\n", myhost);
 
+#ifdef SO_BINDTODEVICE
+    /* I am not sure any other OSs than Linux that provide this feature, and
+     * at the least I cannot test. --Ben
+     *
+     * This feature allows one to tightly bind the local socket to a
+     * particular interface.  This will force even requests to other local
+     * interfaces to go out the external interface.
+     *
+     */
+    if (was_iface) {
+      /* Only bind to the interface when specified as interface, not just as a
+       * hostname or ip address.
+       */
+      if (setsockopt(sockfd, SOL_SOCKET, SO_BINDTODEVICE,
+                     data->set.device, strlen(data->set.device)+1) != 0) {
+        /* printf("Failed to BINDTODEVICE, socket: %d  device: %s error: %s\n",
+           sockfd, data->set.device, strerror(errno)); */
+        infof(data, "SO_BINDTODEVICE %s failed\n",
+              data->set.device);
+        /* This is typiclally "errno 1, error: Operation not permitted" if
+           you're not running as root or another suitable privileged user */
+      }
+    }
+#endif
+
     in=inet_addr(myhost);
     if (CURL_INADDR_NONE != in) {
 
@@ -269,10 +314,11 @@ static CURLcode bindlocal(struct connectdata *conn,
         /* we don't need it anymore after this function has returned */
 
 #ifdef ENABLE_IPV6
-        (void)sa; /* prevent compiler warning */
         if( bind(sockfd, addr->ai_addr, addr->ai_addrlen) >= 0) {
           /* we succeeded to bind */
           struct sockaddr_in6 add;
+
+          bindworked = TRUE;
 	
           size = sizeof(add);
           if(getsockname(sockfd, (struct sockaddr *) &add,
@@ -282,25 +328,31 @@ static CURLcode bindlocal(struct connectdata *conn,
           }
         }
 #else
-        memset((char *)&sa, 0, sizeof(sa));
-        memcpy((char *)&sa.sin_addr, addr->h_addr, addr->h_length);
-        sa.sin_family = AF_INET;
-        sa.sin_addr.s_addr = in;
-        sa.sin_port = 0; /* get any port */
+        {
+          struct sockaddr_in sa;
+
+          memset((char *)&sa, 0, sizeof(sa));
+          memcpy((char *)&sa.sin_addr, addr->h_addr, addr->h_length);
+          sa.sin_family = AF_INET;
+          sa.sin_addr.s_addr = in;
+          sa.sin_port = 0; /* get any port */
 	
-        if( bind(sockfd, (struct sockaddr *)&sa, sizeof(sa)) >= 0) {
-          /* we succeeded to bind */
-          struct sockaddr_in add;
+          if( bind(sockfd, (struct sockaddr *)&sa, sizeof(sa)) >= 0) {
+            /* we succeeded to bind */
+            struct sockaddr_in add;
 	
-          size = sizeof(add);
-          if(getsockname(sockfd, (struct sockaddr *) &add,
-                         (socklen_t *)&size)<0) {
-            failf(data, "getsockname() failed");
-            return CURLE_HTTP_PORT_FAILED;
+            bindworked = TRUE;
+            
+            size = sizeof(add);
+            if(getsockname(sockfd, (struct sockaddr *) &add,
+                           (socklen_t *)&size)<0) {
+              failf(data, "getsockname() failed");
+              return CURLE_HTTP_PORT_FAILED;
+            }
           }
         }
 #endif
-        else {
+        if(!bindworked) {
           switch(errno) {
           case EBADF:
             failf(data, "Invalid descriptor: %d", errno);
@@ -356,7 +408,7 @@ static CURLcode bindlocal(struct connectdata *conn,
 /*
  * verifyconnect() returns TRUE if the connect really has happened.
  */
-static bool verifyconnect(int sockfd)
+static bool verifyconnect(curl_socket_t sockfd)
 {
 #if defined(SO_ERROR) && !defined(WIN32)
   int err = 0;
@@ -372,6 +424,7 @@ static bool verifyconnect(int sockfd)
   /* This wasn't a successful connect */
   return FALSE;
 #else
+  (void)sockfd;
   return TRUE;
 #endif
 }
@@ -382,7 +435,7 @@ static bool verifyconnect(int sockfd)
  */
 
 CURLcode Curl_is_connected(struct connectdata *conn,
-                           int sockfd,
+                           curl_socket_t sockfd,
                            bool *connected)
 {
   int rc;
@@ -460,19 +513,23 @@ CURLcode Curl_is_connected(struct connectdata *conn,
 CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
                           struct Curl_dns_entry *remotehost, /* use this one */
                           int port,                  /* connect to this */
-                          int *sockconn,             /* the connected socket */
+                          curl_socket_t *sockconn,   /* the connected socket */
                           Curl_ipconnect **addr,     /* the one we used */
                           bool *connected)           /* really connected? */
 {
   struct SessionHandle *data = conn->data;
   int rc;
-  int sockfd=-1;
+  curl_socket_t sockfd= CURL_SOCKET_BAD;
   int aliasindex=0;
   char *hostname;
 
   struct timeval after;
   struct timeval before = Curl_tvnow();
 
+#ifdef ENABLE_IPV6
+  struct addrinfo *ai;
+#endif  
+
   /*************************************************************
    * Figure out what maximum time we have left
    *************************************************************/
@@ -513,118 +570,21 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
   }
 
   hostname = data->change.proxy?conn->proxyhost:conn->hostname;
-  infof(data, "About to connect() to %s%s%s:%d\n",
-        conn->bits.ipv6_ip?"[":"",
-        hostname,
-        conn->bits.ipv6_ip?"]":"",
-        port);
+  infof(data, "About to connect() to %s port %d\n",
+        hostname, port);
 
 #ifdef ENABLE_IPV6
   /*
-   * Connecting with IPv6 support is so much easier and cleanly done
+   * Connecting with a getaddrinfo chain
    */
-  {
-    struct addrinfo *ai;
-    port =0; /* prevent compiler warning */
-
-    for (ai = remotehost->addr; ai; ai = ai->ai_next, aliasindex++) {
-      sockfd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
-      if (sockfd < 0)
-        continue;
-
-      if(conn->data->set.device) {
-        /* user selected to bind the outgoing socket to a specified "device"
-           before doing connect */
-        CURLcode res = bindlocal(conn, sockfd);
-        if(res)
-          return res;
-      }
-
-      /* set socket non-blocking */
-      Curl_nonblock(sockfd, TRUE);
-
-      rc = connect(sockfd, ai->ai_addr, ai->ai_addrlen);
-
-      if(-1 == rc) {
-        int error=Curl_ourerrno();
-
-        switch (error) {
-        case EINPROGRESS:
-        case EWOULDBLOCK:
-#if defined(EAGAIN) && EAGAIN != EWOULDBLOCK
-          /* On some platforms EAGAIN and EWOULDBLOCK are the
-           * same value, and on others they are different, hence
-           * the odd #if
-           */
-        case EAGAIN:
-#endif
-        case EINTR:
-          /* asynchronous connect, wait for connect or timeout */
-          if(data->state.used_interface == Curl_if_multi)
-            /* don't hang when doing multi */
-            timeout_ms = 0;
-
-          rc = waitconnect(sockfd, timeout_ms);
-          break;
-        case ECONNREFUSED: /* no one listening */
-        default:
-          /* unknown error, fallthrough and try another address! */
-          failf(data, "Failed connect to %s: %d", hostname, error);
-          break;
-        }
-      }
-
-      if(0 == rc) {
-        /* we might be connected, if the socket says it is OK! Ask it! */
-        if(verifyconnect(sockfd)) {
-          /* we are connected, awesome! */
-          *connected = TRUE; /* this is truly a connect */
-          break;
-	}
-        failf(data, "socket error");
-        /* we are _not_ connected, it was a false alert, continue please */
-      }
-      else if(2 == rc)
-        /* waitconnect() returned error */
-        ;
-      else if(data->state.used_interface == Curl_if_multi) {
-        /* When running the multi interface, we bail out here */
-        rc = 0;
-        break;
-      }
-
-      /* connect failed or timed out */
-      sclose(sockfd);
-      sockfd = -1;
-
-      /* get a new timeout for next attempt */
-      after = Curl_tvnow();
-      timeout_ms -= Curl_tvdiff(after, before);
-      if(timeout_ms < 0) {
-        failf(data, "connect() timed out!");
-        return CURLE_OPERATION_TIMEOUTED;
-      }
-      before = after;
+  for (ai = remotehost->addr; ai; ai = ai->ai_next, aliasindex++) {
+    sockfd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+    if (sockfd == CURL_SOCKET_BAD)
       continue;
-    }
-    if (sockfd < 0)
-      return CURLE_COULDNT_CONNECT;
-
-    /* leave the socket in non-blocking mode */
-
-    if(addr)
-      *addr = ai; /* the address we ended up connected to */
-  }
 #else
   /*
-   * Connecting with IPv4-only support
+   * Connecting with old style IPv4-only support
    */
-  if(!remotehost->addr->h_addr_list[0]) {
-    /* If there is no addresses in the address list, then we return
-       error right away */
-    failf(data, "no address available");
-    return CURLE_COULDNT_CONNECT;
-  }
 
   /* This is the loop that attempts to connect to all IP-addresses we
      know for the given host. One by one. */
@@ -635,11 +595,20 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
 
     /* create an IPv4 TCP socket */
     sockfd = socket(AF_INET, SOCK_STREAM, 0);
-    if(-1 == sockfd) {
+    if(CURL_SOCKET_BAD == sockfd) {
       failf(data, "couldn't create socket");
       return CURLE_COULDNT_CONNECT; /* big time error */
     }
-  
+
+    /* nasty address work before connect can be made */
+    memset((char *) &serv_addr, '\0', sizeof(serv_addr));
+    memcpy((char *)&(serv_addr.sin_addr),
+           (struct in_addr *)remotehost->addr->h_addr_list[aliasindex],
+           sizeof(struct in_addr));
+    serv_addr.sin_family = remotehost->addr->h_addrtype;
+    serv_addr.sin_port = htons((unsigned short)port);
+#endif
+
     if(conn->data->set.device) {
       /* user selected to bind the outgoing socket to a specified "device"
          before doing connect */
@@ -648,19 +617,17 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
         return res;
     }
 
-    /* Convert socket to non-blocking type */
+    /* set socket non-blocking */
     Curl_nonblock(sockfd, TRUE);
 
-    /* do this nasty work to do the connect */
-    memset((char *) &serv_addr, '\0', sizeof(serv_addr));
-    memcpy((char *)&(serv_addr.sin_addr),
-           (struct in_addr *)remotehost->addr->h_addr_list[aliasindex],
-           sizeof(struct in_addr));
-    serv_addr.sin_family = remotehost->addr->h_addrtype;
-    serv_addr.sin_port = htons((unsigned short)port);
-  
-    rc = connect(sockfd, (struct sockaddr *)&serv_addr,
-                 sizeof(serv_addr));
+    /* do not use #ifdef within the function arguments below, as connect() is
+       a defined macro on some platforms and some compilers don't like to mix
+       #ifdefs with macro usage! (AmigaOS is one such platform) */
+#ifdef ENABLE_IPV6
+    rc = connect(sockfd, ai->ai_addr, ai->ai_addrlen);
+#else
+    rc = connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
+#endif
 
     if(-1 == rc) {
       int error=Curl_ourerrno();
@@ -679,7 +646,7 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
         if(data->state.used_interface == Curl_if_multi)
           /* don't hang when doing multi */
           timeout_ms = 0;
-
+        
         rc = waitconnect(sockfd, timeout_ms);
         break;
       default:
@@ -698,7 +665,7 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
       rc = 0;
       break;
     }
-
+      
     if(0 == rc) {
       if (verifyconnect(sockfd)) {
         /* we are connected, awesome! */
@@ -709,22 +676,20 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
       rc = -1;
     }
 
-    if(0 != rc) {
-      /* get a new timeout for next attempt */
-      sclose(sockfd);
-      after = Curl_tvnow();
-      timeout_ms -= Curl_tvdiff(after, before);
-      if(timeout_ms < 0) {
-        failf(data, "Connect timeout on IP number %d", aliasindex+1);
-        break;
-      }
-      before = after;
-      continue; /* try next address */
-    }
-    break;
-  }
+    /* connect failed or timed out */
+    sclose(sockfd);
+    sockfd = -1;
 
-  if(0 != rc) {
+    /* get a new timeout for next attempt */
+    after = Curl_tvnow();
+    timeout_ms -= Curl_tvdiff(after, before);
+    if(timeout_ms < 0) {
+      failf(data, "connect() timed out!");
+      return CURLE_OPERATION_TIMEOUTED;
+    }
+    before = after;
+  }
+  if (sockfd == CURL_SOCKET_BAD) {
     /* no good connect was made */
     *sockconn = -1;
     failf(data, "Connect failed");
@@ -733,10 +698,14 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
 
   /* leave the socket in non-blocking mode */
 
-  if(addr)
-    /* this is the address we've connected to */
+  /* store the address we use */
+  if(addr) {
+#ifdef ENABLE_IPV6
+    *addr = ai;
+#else
     *addr = (struct in_addr *)remotehost->addr->h_addr_list[aliasindex];
 #endif
+  }
 
   /* allow NULL-pointers to get passed in */
   if(sockconn)
@@ -744,4 +713,3 @@ CURLcode Curl_connecthost(struct connectdata *conn,  /* context */
 
   return CURLE_OK;
 }
-
