@@ -29,6 +29,9 @@
 **  a couple of people on Usenet.  Completely overhauled by Rich $alz
 **  <rsalz@bbn.com> and Jim Berets <jberets@bbn.com> in August, 1990.
 **
+**  This code has been modified since it was included in curl, to make it
+**  thread-safe and to make compilers complain less about it.
+**
 **  This code is in the public domain and has no copyright.
 */
 
@@ -173,9 +176,6 @@
 #define yytable  Curl_gd_yytable
 #define yycheck  Curl_gd_yycheck
 
-static int yylex ();
-static int yyerror ();
-
 #define EPOCH		1970
 #define HOUR(x)		((x) * 60)
 
@@ -236,6 +236,10 @@ typedef union {
     int			Number;
     enum _MERIDIAN	Meridian;
 } YYSTYPE;
+#line 224 "getdate.y"
+
+static int yylex (YYSTYPE *yylval, void *cookie);
+static int yyerror (const char *s);
 #include <stdio.h>
 
 #ifndef __cplusplus
@@ -314,11 +318,11 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   235,   236,   239,   242,   245,   248,   251,   254,   257,   263,
-   269,   278,   284,   296,   299,   302,   308,   312,   316,   322,
-   326,   344,   350,   356,   360,   365,   369,   376,   384,   387,
-   390,   393,   396,   399,   402,   405,   408,   411,   414,   417,
-   420,   423,   426,   429,   432,   435,   438,   443,   477,   481
+   240,   241,   244,   247,   250,   253,   256,   259,   262,   268,
+   274,   283,   289,   301,   304,   307,   313,   317,   321,   327,
+   331,   349,   355,   361,   365,   370,   374,   381,   389,   392,
+   395,   398,   401,   404,   407,   410,   413,   416,   419,   422,
+   425,   428,   431,   434,   437,   440,   443,   448,   482,   486
 };
 #endif
 
@@ -944,37 +948,37 @@ yyreduce:
   switch (yyn) {
 
 case 3:
-#line 239 "getdate.y"
+#line 244 "getdate.y"
 {
 	    context->yyHaveTime++;
 	;
     break;}
 case 4:
-#line 242 "getdate.y"
+#line 247 "getdate.y"
 {
 	    context->yyHaveZone++;
 	;
     break;}
 case 5:
-#line 245 "getdate.y"
+#line 250 "getdate.y"
 {
 	    context->yyHaveDate++;
 	;
     break;}
 case 6:
-#line 248 "getdate.y"
+#line 253 "getdate.y"
 {
 	    context->yyHaveDay++;
 	;
     break;}
 case 7:
-#line 251 "getdate.y"
+#line 256 "getdate.y"
 {
 	    context->yyHaveRel++;
 	;
     break;}
 case 9:
-#line 257 "getdate.y"
+#line 262 "getdate.y"
 {
 	    context->yyHour = yyvsp[-1].Number;
 	    context->yyMinutes = 0;
@@ -983,7 +987,7 @@ case 9:
 	;
     break;}
 case 10:
-#line 263 "getdate.y"
+#line 268 "getdate.y"
 {
 	    context->yyHour = yyvsp[-3].Number;
 	    context->yyMinutes = yyvsp[-1].Number;
@@ -992,7 +996,7 @@ case 10:
 	;
     break;}
 case 11:
-#line 269 "getdate.y"
+#line 274 "getdate.y"
 {
 	    context->yyHour = yyvsp[-3].Number;
 	    context->yyMinutes = yyvsp[-1].Number;
@@ -1004,7 +1008,7 @@ case 11:
 	;
     break;}
 case 12:
-#line 278 "getdate.y"
+#line 283 "getdate.y"
 {
 	    context->yyHour = yyvsp[-5].Number;
 	    context->yyMinutes = yyvsp[-3].Number;
@@ -1013,7 +1017,7 @@ case 12:
 	;
     break;}
 case 13:
-#line 284 "getdate.y"
+#line 289 "getdate.y"
 {
 	    context->yyHour = yyvsp[-5].Number;
 	    context->yyMinutes = yyvsp[-3].Number;
@@ -1026,53 +1030,53 @@ case 13:
 	;
     break;}
 case 14:
-#line 296 "getdate.y"
+#line 301 "getdate.y"
 {
 	    context->yyTimezone = yyvsp[0].Number;
 	;
     break;}
 case 15:
-#line 299 "getdate.y"
+#line 304 "getdate.y"
 {
 	    context->yyTimezone = yyvsp[0].Number - 60;
 	;
     break;}
 case 16:
-#line 303 "getdate.y"
+#line 308 "getdate.y"
 {
 	    context->yyTimezone = yyvsp[-1].Number - 60;
 	;
     break;}
 case 17:
-#line 308 "getdate.y"
+#line 313 "getdate.y"
 {
 	    context->yyDayOrdinal = 1;
 	    context->yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 18:
-#line 312 "getdate.y"
+#line 317 "getdate.y"
 {
 	    context->yyDayOrdinal = 1;
 	    context->yyDayNumber = yyvsp[-1].Number;
 	;
     break;}
 case 19:
-#line 316 "getdate.y"
+#line 321 "getdate.y"
 {
 	    context->yyDayOrdinal = yyvsp[-1].Number;
 	    context->yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 20:
-#line 322 "getdate.y"
+#line 327 "getdate.y"
 {
 	    context->yyMonth = yyvsp[-2].Number;
 	    context->yyDay = yyvsp[0].Number;
 	;
     break;}
 case 21:
-#line 326 "getdate.y"
+#line 331 "getdate.y"
 {
 	  /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as MM/DD/YY.
 	     The goal in recognizing YYYY/MM/DD is solely to support legacy
@@ -1093,7 +1097,7 @@ case 21:
 	;
     break;}
 case 22:
-#line 344 "getdate.y"
+#line 349 "getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    context->yyYear = yyvsp[-2].Number;
@@ -1102,7 +1106,7 @@ case 22:
 	;
     break;}
 case 23:
-#line 350 "getdate.y"
+#line 355 "getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    context->yyDay = yyvsp[-2].Number;
@@ -1111,14 +1115,14 @@ case 23:
 	;
     break;}
 case 24:
-#line 356 "getdate.y"
+#line 361 "getdate.y"
 {
 	    context->yyMonth = yyvsp[-1].Number;
 	    context->yyDay = yyvsp[0].Number;
 	;
     break;}
 case 25:
-#line 360 "getdate.y"
+#line 365 "getdate.y"
 {
 	    context->yyMonth = yyvsp[-3].Number;
 	    context->yyDay = yyvsp[-2].Number;
@@ -1126,14 +1130,14 @@ case 25:
 	;
     break;}
 case 26:
-#line 365 "getdate.y"
+#line 370 "getdate.y"
 {
 	    context->yyMonth = yyvsp[0].Number;
 	    context->yyDay = yyvsp[-1].Number;
 	;
     break;}
 case 27:
-#line 369 "getdate.y"
+#line 374 "getdate.y"
 {
 	    context->yyMonth = yyvsp[-1].Number;
 	    context->yyDay = yyvsp[-2].Number;
@@ -1141,7 +1145,7 @@ case 27:
 	;
     break;}
 case 28:
-#line 376 "getdate.y"
+#line 381 "getdate.y"
 {
 	    context->yyRelSeconds = -context->yyRelSeconds;
 	    context->yyRelMinutes = -context->yyRelMinutes;
@@ -1152,115 +1156,115 @@ case 28:
 	;
     break;}
 case 30:
-#line 387 "getdate.y"
+#line 392 "getdate.y"
 {
 	    context->yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 31:
-#line 390 "getdate.y"
+#line 395 "getdate.y"
 {
 	    context->yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 32:
-#line 393 "getdate.y"
+#line 398 "getdate.y"
 {
 	    context->yyRelYear += yyvsp[0].Number;
 	;
     break;}
 case 33:
-#line 396 "getdate.y"
+#line 401 "getdate.y"
 {
 	    context->yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 34:
-#line 399 "getdate.y"
+#line 404 "getdate.y"
 {
 	    context->yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 35:
-#line 402 "getdate.y"
+#line 407 "getdate.y"
 {
 	    context->yyRelMonth += yyvsp[0].Number;
 	;
     break;}
 case 36:
-#line 405 "getdate.y"
+#line 410 "getdate.y"
 {
 	    context->yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 37:
-#line 408 "getdate.y"
+#line 413 "getdate.y"
 {
 	    context->yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 38:
-#line 411 "getdate.y"
+#line 416 "getdate.y"
 {
 	    context->yyRelDay += yyvsp[0].Number;
 	;
     break;}
 case 39:
-#line 414 "getdate.y"
+#line 419 "getdate.y"
 {
 	    context->yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 40:
-#line 417 "getdate.y"
+#line 422 "getdate.y"
 {
 	    context->yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 41:
-#line 420 "getdate.y"
+#line 425 "getdate.y"
 {
 	    context->yyRelHour += yyvsp[0].Number;
 	;
     break;}
 case 42:
-#line 423 "getdate.y"
+#line 428 "getdate.y"
 {
 	    context->yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 43:
-#line 426 "getdate.y"
+#line 431 "getdate.y"
 {
 	    context->yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 44:
-#line 429 "getdate.y"
+#line 434 "getdate.y"
 {
 	    context->yyRelMinutes += yyvsp[0].Number;
 	;
     break;}
 case 45:
-#line 432 "getdate.y"
+#line 437 "getdate.y"
 {
 	    context->yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 46:
-#line 435 "getdate.y"
+#line 440 "getdate.y"
 {
 	    context->yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 47:
-#line 438 "getdate.y"
+#line 443 "getdate.y"
 {
 	    context->yyRelSeconds += yyvsp[0].Number;
 	;
     break;}
 case 48:
-#line 444 "getdate.y"
+#line 449 "getdate.y"
 {
 	    if (context->yyHaveTime && context->yyHaveDate &&
 		!context->yyHaveRel)
@@ -1294,13 +1298,13 @@ case 48:
 	  ;
     break;}
 case 49:
-#line 478 "getdate.y"
+#line 483 "getdate.y"
 {
 	    yyval.Meridian = MER24;
 	  ;
     break;}
 case 50:
-#line 482 "getdate.y"
+#line 487 "getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	  ;
@@ -1527,7 +1531,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 487 "getdate.y"
+#line 492 "getdate.y"
 
 
 /* Include this file down here because bison inserts code above which
@@ -1536,9 +1540,9 @@ yyerrhandle:
 #include "getdate.h"
 
 #ifndef WIN32 /* the windows dudes don't need these, does anyone really? */
-extern struct tm	*gmtime ();
-extern struct tm	*localtime ();
-extern time_t		mktime ();
+extern struct tm	*gmtime (const time_t *);
+extern struct tm	*localtime (const time_t *);
+extern time_t		mktime (struct tm *);
 #endif
 
 /* Month and day table. */
@@ -1730,16 +1734,13 @@ static TABLE const MilitaryTable[] = {
 
 /* ARGSUSED */
 static int
-yyerror (s)
-     char *s ATTRIBUTE_UNUSED;
+yyerror (const char *s ATTRIBUTE_UNUSED)
 {
   return 0;
 }
 
 static int
-ToHour (Hours, Meridian)
-     int Hours;
-     MERIDIAN Meridian;
+ToHour (int Hours, MERIDIAN Meridian)
 {
   switch (Meridian)
     {
@@ -1766,8 +1767,7 @@ ToHour (Hours, Meridian)
 }
 
 static int
-ToYear (Year)
-     int Year;
+ToYear (int Year)
 {
   if (Year < 0)
     Year = -Year;
@@ -1783,9 +1783,7 @@ ToYear (Year)
 }
 
 static int
-LookupWord (yylval, buff)
-     YYSTYPE *yylval;
-     char *buff;
+LookupWord (YYSTYPE *yylval, char *buff)
 {
   register char *p;
   register char *q;
@@ -1905,9 +1903,7 @@ LookupWord (yylval, buff)
 }
 
 static int
-yylex (yylval, cookie)
-     YYSTYPE *yylval;
-     void *cookie;
+yylex (YYSTYPE *yylval, void *cookie)
 {
   register unsigned char c;
   register char *p;
@@ -2126,9 +2122,7 @@ curl_getdate (const char *p, const time_t *now)
 
 /* ARGSUSED */
 int
-main (ac, av)
-     int ac;
-     char *av[];
+main (int ac, char *av[])
 {
   char buff[MAX_BUFF_LEN + 1];
   time_t d;
