@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http_ntlm.c,v 1.23 2003/10/17 12:44:54 bagder Exp $
+ * $Id: http_ntlm.c,v 1.28 2004/03/08 16:20:51 bagder Exp $
  ***************************************************************************/
 #include "setup.h"
 
@@ -126,7 +126,7 @@ CURLntlm Curl_input_ntlm(struct connectdata *conn,
          32 (48) start of data block
       */
 
-      int size = Curl_base64_decode(header, buffer);
+      size_t size = Curl_base64_decode(header, (char *)buffer);
 
       ntlm->state = NTLMSTATE_TYPE2; /* we got a type-2 */
 
@@ -212,8 +212,8 @@ static void mkhash(char *password,
   static const unsigned char magic[] = {
     0x4B, 0x47, 0x53, 0x21, 0x40, 0x23, 0x24, 0x25
   };
-  int i;
-  int len = strlen(password);
+  unsigned int i;
+  size_t len = strlen(password);
 
   /* make it fit at least 14 bytes */
   pw = malloc(len<7?14:len*2);
@@ -282,11 +282,11 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
 {
   const char *domain=""; /* empty */
   const char *host=""; /* empty */
-  int domlen=strlen(domain);
-  int hostlen = strlen(host);
+  int domlen=(int)strlen(domain);
+  int hostlen = (int)strlen(host);
   int hostoff; /* host name offset */
   int domoff;  /* domain name offset */
-  int size;
+  size_t size;
   char *base64=NULL;
   unsigned char ntlmbuf[256]; /* enough, unless the host/domain is very long */
 
@@ -376,7 +376,7 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
     size = 32 + hostlen + domlen;
 
     /* now keeper of the base64 encoded package size */
-    size = Curl_base64_encode(ntlmbuf, size, &base64);
+    size = Curl_base64_encode((char *)ntlmbuf, size, &base64);
 
     if(size >0 ) {
       Curl_safefree(*allocuserpwd);
@@ -549,7 +549,7 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
     ntlmbuf[57] = size >> 8;
 
     /* convert the binary blob into base64 */
-    size = Curl_base64_encode(ntlmbuf, size, &base64);
+    size = Curl_base64_encode((char *)ntlmbuf, size, &base64);
 
     if(size >0 ) {
       Curl_safefree(*allocuserpwd);
