@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: transfer.c,v 1.182 2003/10/24 21:54:34 bagder Exp $
+ * $Id: transfer.c,v 1.212 2004/03/16 09:16:38 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -29,11 +29,14 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #include <sys/stat.h>
 
 #include <errno.h>
 
+#include "strtoofft.h"
 #include "strequal.h"
 
 #if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
@@ -43,7 +46,9 @@
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
 #include <sys/time.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -55,7 +60,9 @@
 #ifdef HAVE_NET_IF_H
 #include <net/if.h>
 #endif
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
+#endif
 #include <signal.h>
 
 #ifdef HAVE_SYS_PARAM_H
@@ -80,8 +87,7 @@
 #include <curl/types.h>
 #include "netrc.h"
 
-#include "content_encoding.h"   /* content encoding support. 08/27/02 jhrg */
-
+#include "content_encoding.h"
 #include "hostip.h"
 #include "transfer.h"
 #include "sendf.h"
@@ -105,10 +111,6 @@
 #include "memdebug.h"
 #endif
 
-#ifndef min
-#define min(a, b)   ((a) < (b) ? (a) : (b))
-#endif
-
 #define CURL_TIMEOUT_EXPECT_100 1000 /* counting ms here */
 
 enum {
@@ -151,16 +153,16 @@ static int fillbuffer(struct connectdata *conn,
 
     /* copy the prefix to the buffer */
     memcpy(conn->upload_fromhere, hexbuffer, hexlen);
-    if(nread>hexlen) {
-      /* append CRLF to the data */
-      memcpy(conn->upload_fromhere +
-             nread, "\r\n", 2);
-      nread+=2;
-    }
-    else {
+
+    /* always append CRLF to the data */
+    memcpy(conn->upload_fromhere + nread, "\r\n", 2);
+
+    if((nread - hexlen) == 0) {
       /* mark this as done once this chunk is transfered */
       conn->keep.upload_done = TRUE;
     }
+
+    nread+=2; /* for the added CRLF */
   }
   return nread;
 }
@@ -193,7 +195,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
 {
   struct Curl_transfer_keeper *k = &conn->keep;
   struct SessionHandle *data = conn->data;
-  int result;
+  CURLcode result;
   ssize_t nread; /* number of bytes read */
   int didwhat=0;
   
@@ -206,6 +208,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
 
   fd_set *readfdp = k->readfdp;
   fd_set *writefdp = k->writefdp;
+  curl_off_t contentlength;
   
   if((k->keepon & KEEP_READ) && !readfdp) {
     /* reading is requested, but no socket descriptor pointer was set */
@@ -243,10 +246,15 @@ CURLcode Curl_readwrite(struct connectdata *conn,
           data->set.buffer_size:BUFSIZE -1;
 
         /* receive data from the network! */
-        result = Curl_read(conn, conn->sockfd, k->buf, buffersize, &nread);
+        int readrc = Curl_read(conn, conn->sockfd, k->buf, buffersize, &nread);
 
-        if(0>result)
+        /* subzero, this would've blocked */
+        if(0>readrc)
           break; /* get out of loop */
+
+        /* get the CURLcode from the int */
+        result = (CURLcode)readrc;
+
         if(result>0)
           return result;
 
@@ -301,8 +309,8 @@ CURLcode Curl_readwrite(struct connectdata *conn,
               if (k->hbuflen + nread >= data->state.headersize) {
                 /* We enlarge the header buffer as it is too small */
                 char *newbuff;
-                long newsize=MAX((k->hbuflen+nread)*3/2,
-                                 data->state.headersize*2);
+                long newsize=CURLMAX((k->hbuflen+nread)*3/2,
+                                     data->state.headersize*2);
                 hbufp_index = k->hbufp - data->state.headerbuff;
                 newbuff = (char *)realloc(data->state.headerbuff, newsize);
                 if(!newbuff) {
@@ -346,8 +354,8 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             if (k->hbuflen + full_length >=
                 data->state.headersize) {
               char *newbuff;
-              long newsize=MAX((k->hbuflen+full_length)*3/2,
-                               data->state.headersize*2);
+              long newsize=CURLMAX((k->hbuflen+full_length)*3/2,
+                                   data->state.headersize*2);
               hbufp_index = k->hbufp - data->state.headerbuff;
               newbuff = (char *)realloc(data->state.headerbuff, newsize);
               if(!newbuff) {
@@ -450,10 +458,28 @@ CURLcode Curl_readwrite(struct connectdata *conn,
               data->info.header_size += headerlen;
               conn->headerbytecount += headerlen;
 
-              /* *auth_act() checks what authentication methods that are
-                 available and decides which one (if any) to use. It will
-                 set 'newurl' if an auth metod was picked. */
-              Curl_http_auth_act(conn);
+              if (conn->resume_from &&
+                  !k->content_range &&
+                  (data->set.httpreq==HTTPREQ_GET)) {
+                if(k->httpcode == 416) {
+                  /* "Requested Range Not Satisfiable" */
+                  stop_reading = TRUE;
+                }
+                else {
+                  /* we wanted to resume a download, although the server
+                     doesn't seem to support this and we did this with a GET
+                     (if it wasn't a GET we did a POST or PUT resume) */
+                  failf (data, "HTTP server doesn't seem to support "
+                         "byte ranges. Cannot resume.");
+                  return CURLE_HTTP_RANGE_ERROR;
+                }
+              }
+
+              if(!stop_reading)
+                /* *auth_act() checks what authentication methods that are
+                   available and decides which one (if any) to use. It will
+                   set 'newurl' if an auth metod was picked. */
+                Curl_http_auth_act(conn);
               
               if(!k->header) {
                 /*
@@ -474,8 +500,17 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                      "Content-Length: 0" still prevents us from attempting to
                      read the (missing) response-body.
                   */
-                  if(-1 != conn->size)
+                  /* According to RFC2616 section 4.4, we MUST ignore
+                     Content-Length: headers if we are now receiving data
+                     using chunked Transfer-Encoding.
+                  */
+                  if(conn->bits.chunk)
+                    conn->size=-1;
+
+                  if(-1 != conn->size) {
+                    Curl_pgrsSetDownloadSize(data, conn->size);
                     conn->maxdownload = conn->size;
+                  }
                 }
                 /* If max download size is *zero* (nothing) we already
                    have nothing and can safely return ok now! */
@@ -589,15 +624,14 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                resuming a document that we don't get, and this header contains 
                info about the true size of the document we didn't get now. */
             if ((k->httpcode != 416) &&
-                checkprefix("Content-Length:", k->p) &&
-                sscanf (k->p+15, " %ld", &k->contentlength)) {
-              if (data->set.max_filesize && k->contentlength > 
+                checkprefix("Content-Length:", k->p)) {
+              contentlength = strtoofft(k->p+15, NULL, 10);
+              if (data->set.max_filesize && contentlength > 
                   data->set.max_filesize) {
                 failf(data, "Maximum file size exceeded");
                 return CURLE_FILESIZE_EXCEEDED;
               }
-              conn->size = k->contentlength;
-              Curl_pgrsSetDownloadSize(data, k->contentlength);
+              conn->size = contentlength;
             }
             /* check for Content-Type: header lines to get the mime-type */
             else if (checkprefix("Content-Type:", k->p)) {
@@ -697,7 +731,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                   *start && isspace((int)*start);
                   start++);
 
-              /* Record the content-encoding for later use. 08/27/02 jhrg */
+              /* Record the content-encoding for later use */
               if (checkprefix("identity", start))
                 k->content_encoding = IDENTITY;
               else if (checkprefix("deflate", start))
@@ -709,17 +743,26 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                        || checkprefix("x-compress", start))
                 k->content_encoding = COMPRESS;
             }
-            else if (checkprefix("Content-Range:", k->p)) {
-              if (sscanf (k->p+14, " bytes %d-", &k->offset) ||
-                  sscanf (k->p+14, " bytes: %d-", &k->offset)) {
-                /* This second format was added August 1st 2000 by Igor
-                   Khristophorov since Sun's webserver JavaWebServer/1.1.1
-                   obviously sends the header this way! :-( */
-                if (conn->resume_from == k->offset) {
-                  /* we asked for a resume and we got it */
-                  k->content_range = TRUE;
-                }
-              }
+            else if (Curl_compareheader(k->p, "Content-Range:", "bytes")) {
+              /* Content-Range: bytes [num]-
+                 Content-Range: bytes: [num]-
+
+                 The second format was added August 1st 2000 by Igor
+                 Khristophorov since Sun's webserver JavaWebServer/1.1.1
+                 obviously sends the header this way! :-( */
+
+              char *ptr = strstr(k->p, "bytes");
+              ptr+=5;
+
+              if(*ptr == ':')
+                /* stupid colon skip */
+                ptr++;
+
+              k->offset = strtoofft(ptr, NULL, 10);
+              
+              if (conn->resume_from == k->offset)
+                /* we asked for a resume and we got it */
+                k->content_range = TRUE;
             }
             else if(data->cookies &&
                     checkprefix("Set-Cookie:", k->p)) {
@@ -852,24 +895,14 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                 k->ignorebody = TRUE;
                 infof(data, "Ignoring the response-body\n");
               }
-              if (conn->resume_from &&
-                       !k->content_range &&
-                       (data->set.httpreq==HTTPREQ_GET)) {
-                /* we wanted to resume a download, although the server
-                   doesn't seem to support this and we did this with a GET
-                   (if it wasn't a GET we did a POST or PUT resume) */
-                failf (data, "HTTP server doesn't seem to support "
-                       "byte ranges. Cannot resume.");
-                return CURLE_HTTP_RANGE_ERROR;
-              }
-              else if(data->set.timecondition && !conn->range) {
+              if(data->set.timecondition && !conn->range) {
                 /* A time condition has been set AND no ranges have been
                    requested. This seems to be what chapter 13.3.4 of
                    RFC 2616 defines to be the correct action for a
                    HTTP/1.1 client */
                 if((k->timeofdoc > 0) && (data->set.timevalue > 0)) {
                   switch(data->set.timecondition) {
-                  case TIMECOND_IFMODSINCE:
+                  case CURL_TIMECOND_IFMODSINCE:
                   default:
                     if(k->timeofdoc < data->set.timevalue) {
                       infof(data,
@@ -878,7 +911,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                       return CURLE_OK;
                     }
                     break;
-                  case TIMECOND_IFUNMODSINCE:
+                  case CURL_TIMECOND_IFUNMODSINCE:
                     if(k->timeofdoc > data->set.timevalue) {
                       infof(data,
                             "The requested document is not old enough\n");
@@ -938,7 +971,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
 
           if((-1 != conn->maxdownload) &&
              (k->bytecount + nread >= conn->maxdownload)) {
-            nread = conn->maxdownload - k->bytecount;
+            nread = (ssize_t) (conn->maxdownload - k->bytecount);
             if(nread < 0 ) /* this should be unusual */
               nread = 0;
 
@@ -948,7 +981,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
 
           k->bytecount += nread;
 
-          Curl_pgrsSetDownloadCounter(data, (double)k->bytecount);
+          Curl_pgrsSetDownloadCounter(data, k->bytecount);
             
           if(!conn->bits.chunk && (nread || k->badheader)) {
             /* If this is chunky transfer, it was already written */
@@ -963,7 +996,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
             if(k->badheader < HEADER_ALLBAD) {
               /* This switch handles various content encodings. If there's an
                  error here, be sure to check over the almost identical code
-                 in http_chunks.c. 08/29/02 jhrg
+                 in http_chunks.c.
                  Make sure that ALL_CONTENT_ENCODINGS contains all the
                  encodings handled here. */
 #ifdef HAVE_LIBZ
@@ -972,8 +1005,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
 #endif
                 /* This is the default when the server sends no
                    Content-Encoding header. See Curl_readwrite_init; the
-                   memset() call initializes k->content_encoding to zero.
-                   08/28/02 jhrg */
+                   memset() call initializes k->content_encoding to zero. */
                 if(!k->ignorebody)
                   result = Curl_client_write(data, CLIENTWRITE_BODY, k->str, 
                                              nread);
@@ -990,7 +1022,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                 result = Curl_unencode_gzip_write(data, k, nread);
                 break;
 
-              case COMPRESS:          /* FIXME 08/27/02 jhrg */
+              case COMPRESS:
               default:
                 failf (data, "Unrecognized content encoding type. "
                        "libcurl understands `identity', `deflate' and `gzip' "
@@ -1152,7 +1184,7 @@ CURLcode Curl_readwrite(struct connectdata *conn,
         }
 
         k->writebytecount += bytes_written;
-        Curl_pgrsSetUploadCounter(data, (double)k->writebytecount);
+        Curl_pgrsSetUploadCounter(data, k->writebytecount);
 
       } while(!writedone); /* loop until we're done writing! */
       
@@ -1204,7 +1236,8 @@ CURLcode Curl_readwrite(struct connectdata *conn,
     
   if (data->set.timeout &&
       ((Curl_tvdiff(k->now, k->start)/1000) >= data->set.timeout)) {
-    failf (data, "Operation timed out with %d out of %d bytes received",
+    failf (data, "Operation timed out with %" FORMAT_OFF_T
+           " out of %" FORMAT_OFF_T " bytes received",
            k->bytecount, conn->size);
     return CURLE_OPERATION_TIMEOUTED;
   }
@@ -1215,11 +1248,12 @@ CURLcode Curl_readwrite(struct connectdata *conn,
      * returning.
      */
 
-    if(!(data->set.no_body) && k->contentlength &&
-       (k->bytecount != k->contentlength) &&
+    if(!(data->set.no_body) && (conn->size != -1) &&
+       (k->bytecount != conn->size) &&
        !conn->newurl) {
-      failf(data, "transfer closed with %d bytes remaining to read",
-            k->contentlength-k->bytecount);
+      failf(data, "transfer closed with %" FORMAT_OFF_T
+            " bytes remaining to read",
+            conn->size - k->bytecount);
       return CURLE_PARTIAL_FILE;
     }
     else if(conn->bits.chunk && conn->proto.http->chunk.datasize) {
@@ -1243,7 +1277,7 @@ CURLcode Curl_readwrite_init(struct connectdata *conn)
   struct Curl_transfer_keeper *k = &conn->keep;
 
   /* NB: the content encoding software depends on this initialization of
-     Curl_transfer_keeper. 08/28/02 jhrg */
+     Curl_transfer_keeper. */
   memset(k, 0, sizeof(struct Curl_transfer_keeper));
 
   k->start = Curl_tvnow(); /* start time */
@@ -1274,13 +1308,13 @@ CURLcode Curl_readwrite_init(struct connectdata *conn)
   if(conn->bits.getheader || !data->set.no_body) {
 
     FD_ZERO (&k->readfd);               /* clear it */
-    if(conn->sockfd != -1) {
+    if(conn->sockfd != CURL_SOCKET_BAD) {
       FD_SET (conn->sockfd, &k->readfd); /* read socket */
       k->keepon |= KEEP_READ;
     }
 
     FD_ZERO (&k->writefd);              /* clear it */
-    if(conn->writesockfd != -1) {
+    if(conn->writesockfd != CURL_SOCKET_BAD) {
       /* HTTP 1.1 magic:
 
          Even if we require a 100-return code before uploading data, we might
@@ -1330,7 +1364,10 @@ void Curl_single_fdset(struct connectdata *conn,
   }
   if(conn->keep.keepon & KEEP_WRITE) {
     FD_SET(conn->writesockfd, write_fd_set);
-    if(conn->writesockfd > *max_fd)
+
+    /* since sockets are curl_socket_t nowadays, we typecast it to int here
+       to compare it nicely */
+    if((int)conn->writesockfd > *max_fd)
       *max_fd = conn->writesockfd;
     conn->keep.writefdp = write_fd_set; /* store the address of the set */
   }
@@ -1348,7 +1385,7 @@ void Curl_single_fdset(struct connectdata *conn,
  * The transfer must already have been setup by a call to Curl_Transfer().
  *
  * Note that headers are created in a preallocated buffer of a default size.
- * That buffer can be enlarged on demand, but it is never shrinken again.
+ * That buffer can be enlarged on demand, but it is never shrunken again.
  *
  * Parts of this function was once written by the friendly Mark Butler
  * <butlerm@xmission.com>.
@@ -1362,9 +1399,12 @@ Transfer(struct connectdata *conn)
   struct Curl_transfer_keeper *k = &conn->keep;
   bool done=FALSE;
 
-  Curl_readwrite_init(conn);
+  if(!(conn->protocol & PROT_FILE))
+    /* Only do this if we are not transferring FILE:, since the file: treatment
+       is different*/
+    Curl_readwrite_init(conn);
 
-  if((conn->sockfd == -1) && (conn->writesockfd == -1))
+  if((conn->sockfd == CURL_SOCKET_BAD) && (conn->writesockfd == CURL_SOCKET_BAD))
     /* nothing to read, nothing to write, we're already OK! */
     return CURLE_OK;
 
@@ -1394,9 +1434,6 @@ Transfer(struct connectdata *conn)
         done = TRUE; /* no more read or write */
       continue;
     case 0:  /* timeout */
-      result = Curl_readwrite(conn, &done);
-      break;
-
     default: /* readable descriptors */
       result = Curl_readwrite(conn, &done);
       break;
@@ -1412,20 +1449,19 @@ Transfer(struct connectdata *conn)
 
 CURLcode Curl_pretransfer(struct SessionHandle *data)
 {
-  CURLcode res;
   if(!data->change.url)
     /* we can't do anything wihout URL */
     return CURLE_URL_MALFORMAT;
 
 #ifdef USE_SSLEAY
-  /* Init the SSL session ID cache here. We do it here since we want to
-     do it after the *_setopt() calls (that could change the size) but
-     before any transfer. */
-  res = Curl_SSL_InitSessions(data, data->set.ssl.numsessions);
-  if(res)
-    return res;
-#else
-  (void)res;
+  {
+    /* Init the SSL session ID cache here. We do it here since we want to do
+       it after the *_setopt() calls (that could change the size of the cache)
+       but before any transfer takes place. */
+    CURLcode res = Curl_SSL_InitSessions(data, data->set.ssl.numsessions);
+    if(res)
+      return res;
+  }
 #endif
 
   data->set.followlocation=0; /* reset the location-follow counter */
@@ -1460,7 +1496,7 @@ CURLcode Curl_pretransfer(struct SessionHandle *data)
   * different ports! */
   data->state.allow_port = TRUE;
 
-#if defined(HAVE_SIGNAL) && defined(SIGPIPE)
+#if defined(HAVE_SIGNAL) && defined(SIGPIPE) && !defined(HAVE_MSG_NOSIGNAL)
   /*************************************************************
    * Tell signal handler to ignore SIGPIPE
    *************************************************************/
@@ -1476,10 +1512,12 @@ CURLcode Curl_pretransfer(struct SessionHandle *data)
 
 CURLcode Curl_posttransfer(struct SessionHandle *data)
 {
-#if defined(HAVE_SIGNAL) && defined(SIGPIPE)
+#if defined(HAVE_SIGNAL) && defined(SIGPIPE) && !defined(HAVE_MSG_NOSIGNAL)
   /* restore the signal handler for SIGPIPE before we get back */
   if(!data->set.no_signal)
     signal(SIGPIPE, data->state.prev_signal);
+#else
+  (void)data; /* unused parameter */
 #endif  
 
   return CURLE_OK;
@@ -1658,6 +1696,15 @@ CURLcode Curl_follow(struct SessionHandle *data,
       pathsep = strchr(protsep, '/');
       if(pathsep)
         *pathsep=0;
+      else {
+        /* There was no slash. Now, since we might be operating on a badly
+           formatted URL, such as "http://www.url.com?id=2380" which doesn't
+           use a slash separator as it is supposed to, we need to check for a
+           ?-letter as well! */
+        pathsep = strchr(protsep, '?');
+        if(pathsep)
+          *pathsep=0;
+      }
     }
 
     /* If the new part contains a space, this is a mighty stupid redirect
@@ -1854,10 +1901,9 @@ CURLcode Curl_perform(struct SessionHandle *data)
          to the new URL */
       urlchanged = data->change.url_changed;
       if ((CURLE_OK == res) && urlchanged) {
-        char *gotourl;
         res = Curl_done(conn);
         if(CURLE_OK == res) {
-          newurl = strdup(data->change.url);
+          char *gotourl = strdup(data->change.url);
           res = Curl_follow(data, gotourl);
           if(res)
             free(gotourl);
@@ -1869,31 +1915,45 @@ CURLcode Curl_perform(struct SessionHandle *data)
       res = Curl_do(&conn);
 
       if(res == CURLE_OK) {
-        if(conn->protocol&PROT_FTPS)
-          /* FTPS, disable ssl while transfering data */
-          conn->ssl.use = FALSE;
         res = Transfer(conn); /* now fetch that URL please */
-        if(conn->protocol&PROT_FTPS)
-          /* FTPS, enable ssl again after havving transferred data */
-          conn->ssl.use = TRUE;
+        if(res == CURLE_OK) {
 
-        if(res == CURLE_OK)
-          /*
-           * We must duplicate the new URL here as the connection data
-           * may be free()ed in the Curl_done() function.
-           */
-          newurl = conn->newurl?strdup(conn->newurl):NULL;
+          if((conn->keep.bytecount == 0) &&
+             (conn->sockerror == ECONNRESET) &&
+             conn->bits.reuse) {
+            /* We got no data, the connection was reset and we did attempt
+               to re-use a connection. This smells like we were too fast to
+               re-use a connection that was closed when we wanted to read
+               from it. Bad luck. Let's simulate a redirect to the same URL
+               to retry! */
+            infof(data, "Connection reset, retrying a fresh connect\n");
+            newurl = strdup(conn->data->change.url);
+
+            conn->bits.close = TRUE; /* close this connection */
+            conn->bits.retry = TRUE; /* mark this as a connection we're about
+                                        to retry. Marking it this way should
+                                        prevent i.e HTTP transfers to return
+                                        error just because nothing has been
+                                        transfered! */
+          }
+          else
+            /*
+             * We must duplicate the new URL here as the connection data
+             * may be free()ed in the Curl_done() function.
+             */
+            newurl = conn->newurl?strdup(conn->newurl):NULL;
+        }
         else {
           /* The transfer phase returned error, we mark the connection to get
            * closed to prevent being re-used. This is becasue we can't
            * possibly know if the connection is in a good shape or not now. */
           conn->bits.close = TRUE;
 
-          if(-1 !=conn->secondarysocket) {
+          if(CURL_SOCKET_BAD != conn->sock[SECONDARYSOCKET]) {
             /* if we failed anywhere, we must clean up the secondary socket if
                it was used */
-            sclose(conn->secondarysocket);
-            conn->secondarysocket=-1;
+            sclose(conn->sock[SECONDARYSOCKET]);
+            conn->sock[SECONDARYSOCKET] = CURL_SOCKET_BAD;
           }
         }
 
@@ -1936,13 +1996,14 @@ CURLcode Curl_perform(struct SessionHandle *data)
 
 CURLcode 
 Curl_Transfer(struct connectdata *c_conn, /* connection data */
-              int sockfd,       /* socket to read from or -1 */
-              int size,         /* -1 if unknown at this point */
-              bool getheader,   /* TRUE if header parsing is wanted */
-              long *bytecountp, /* return number of bytes read or NULL */
-              int writesockfd,  /* socket to write to, it may very well be
-                                   the same we read from. -1 disables */
-              long *writebytecountp /* return number of bytes written or
+              int sockindex,       /* socket index to read from or -1 */
+              curl_off_t size,     /* -1 if unknown at this point */
+              bool getheader,      /* TRUE if header parsing is wanted */
+              curl_off_t *bytecountp, /* return number of bytes read or NULL */
+              int writesockindex,  /* socket index to write to, it may very
+                                      well be the same we read from. -1
+                                      disables */
+              curl_off_t *writecountp /* return number of bytes written or
                                        NULL */
               )
 {
@@ -1950,13 +2011,17 @@ Curl_Transfer(struct connectdata *c_conn, /* connection data */
   if(!conn)
     return CURLE_BAD_FUNCTION_ARGUMENT;
 
+  curlassert((sockindex <= 1) && (sockindex >= -1));
+
   /* now copy all input parameters */
-  conn->sockfd = sockfd;
+  conn->sockfd = sockindex==-1?
+    CURL_SOCKET_BAD:conn->sock[sockindex];
   conn->size = size;
   conn->bits.getheader = getheader;
   conn->bytecountp = bytecountp;
-  conn->writesockfd = writesockfd;
-  conn->writebytecountp = writebytecountp;
+  conn->writesockfd = writesockindex==-1?
+    CURL_SOCKET_BAD:conn->sock[writesockindex];
+  conn->writebytecountp = writecountp;
 
   return CURLE_OK;
 
